apc.indiv.fit.model <- function (apc.indiv.design.model, model.family=NULL,
                                 DV=NULL){
  ##################
  # This function estimates the model developed in apc.indiv.design.model
  ##################
  # Step 1: Set up
  # Check model.design
  family.list <- c("gaussian", "binomial")
  if (!isTRUE(model.family %in% family.list))
    stop("model.family must be either 'gaussian' or 'binomial'")
  
  # Extract elements from apc.indiv.design.model
  design <- apc.indiv.design.model$full.design
  if(is.null(DV)) {
    DV <- apc.indiv.design.model$DV}
  xi.dim1 <- apc.indiv.design.model$xi.dim
  model.design <- apc.indiv.design.model$model.design
  
  difdif <- apc.indiv.design.model$difdif
  slopes <- apc.indiv.design.model$slopes
  age.max <- apc.indiv.design.model$age.max
  per.max <- apc.indiv.design.model$per.max
  coh.max <- apc.indiv.design.model$coh.max
  age1 <- apc.indiv.design.model$age1
  per1 <- apc.indiv.design.model$per1
  coh1 <- apc.indiv.design.model$coh1
  unit <- apc.indiv.design.model$unit
  
  if (is.null(DV)) stop("Dependent variable unspecified")
  ##################
  # Step 2: Estimation
  # Regression using glm.fit
  if (model.family == "binomial")
    fit <- glm.fit(design, DV[, 1], family=binomial())
  if (model.family == "gaussian")
    fit <- glm.fit(design, DV[, 1], family=gaussian())
  
  # Extract coefficients and covariance
  coefficients	<- summary.glm(fit)$coefficients
  covariance	<- summary.glm(fit)$cov.scaled
  
  n.coeff <- length(coefficients[, 1])
  n.coeff.canonical <- xi.dim1
  
  # Manipulate coefficients and covariance based on model design
  # First case: model design is 1
  if (model.design == "1"){
    coefficients.canonical	<- t(coefficients[(n.coeff-
                                                n.coeff.canonical+1):n.coeff, ])
    covariance.canonical	<- summary.glm(fit)$cov.scaled[(n.coeff-
                                                           n.coeff.canonical+1):n.coeff, 
                                                        (n.coeff-
                                                           n.coeff.canonical+1):n.coeff]  
    if(n.coeff - n.coeff.canonical > 0)
      coefficients.covariates <- t(coefficients[1:(n.coeff-n.coeff.canonical), ])
    else
      coefficients.covariates <- NULL
    
    #	get standard errors 
    coefficients.canonical[,2]	<- sqrt(covariance.canonical)
    #	get t-statistics
    coefficients.canonical[,3]	<- coefficients.canonical[,1] 	/ 
      coefficients.canonical[,2]
    #	get p-values
    coefficients.canonical[,4]	<- 2*pnorm(abs(coefficients.canonical[  ,3]),
                                          lower.tail=FALSE)
  }
  # Second case: model design is not 1
  if (!model.design == "1"){
    coefficients.canonical	<- summary.glm(fit)$coefficients[(n.coeff-
                                                               n.coeff.canonical+1):n.coeff, ]
    covariance.canonical	<- summary.glm(fit)$cov.scaled[(n.coeff-
                                                           n.coeff.canonical+1):n.coeff, 
                                                        (n.coeff-n.coeff.canonical+1):n.coeff]  
    
    if(n.coeff - n.coeff.canonical > 0)
      coefficients.covariates <- t(coefficients[1:(n.coeff-n.coeff.canonical), ])
    else
      coefficients.covariates <- NULL
    
    #	get standard errors 
    coefficients.canonical[,2]	<- sqrt(diag(covariance.canonical))
    #	get t-statistics
    coefficients.canonical[,3]	<- coefficients.canonical[,1] 	/ 
      coefficients.canonical[,2]
    #	get p-values
    coefficients.canonical[,4]	<- 2*pnorm(abs(coefficients.canonical[  ,3]),
                                          lower.tail=FALSE)
  }
  ##################
  # Step 3: Use output thus far to get dates for use in plotting later 
  
  index.age	<- NULL
  index.per	<- NULL
  index.coh	<- NULL
  
  # changes to index based on max
  start		<- 1+sum(slopes)
  if(difdif[1])	{	index.age	<- start+seq(1,age.max-2);start	<- start+age.max-2	}
  if(difdif[2])	{	index.per	<- start+seq(1,per.max-2);start	<- start+per.max-2	}
  if(difdif[3])	{	index.coh	<- start+seq(1,coh.max-2);start	<- start+coh.max-2	}
  xi.dim2		<- start
  # use index, max, unit, xi.dim2, difdif to get dates
  dates		<- matrix(data=NA,nrow=xi.dim2,ncol=1)			
  if(difdif[1])	dates[index.age,1]	<- age1+seq(2,age.max-1)*unit	
  if(difdif[2])	dates[index.per,1]	<- per1+seq(2,per.max-1)*unit
  if(difdif[3])	dates[index.coh,1]	<- coh1+seq(2,coh.max-1)*unit
  ##################
  # Construct likelihood
  RSS <- sum(fit$residuals^2)
  N <- length(fit$residuals)
  lik <- -(N/2)*log(2*pi*RSS/N) - (N/2)
  # Return valuables
  valuables <- c(fit, list(coefficients.canonical = coefficients.canonical,
                           covariance.canonical = covariance.canonical,
                           dates = dates,
                           index.age = index.age,
                           index.coh = index.coh,
                           index.per = index.per,
                           
                           coefficients.covariates = coefficients.covariates, 
                           
                           model.family = model.family,
                           
                           difdif = difdif,
                           slopes = slopes,
                           age1 = age1,
                           per1 = per1,
                           coh1 = coh1,
                           unit = unit,
                           age.max = age.max,
                           per.max = per.max,
                           coh.max = coh.max,
                           
                           model.design = apc.indiv.design.model$model.design,
                           per.zero = apc.indiv.design.model$per.zero,
                           per.odd = apc.indiv.design.model$per.odd,
                           U = apc.indiv.design.model$U,
                           
                           likelihood = lik))
  return(valuables)
}
